#include <GL/gl.h>
#include <GL/glu.h>

#include "../include/brick.h"

Brick::Brick()
{
    x = 0;
    y = 0;
    z = 0;
    next = NULL;
    prev = NULL;
}

Brick::Brick(GLdouble X, GLdouble Y, GLdouble Z, Brick * n, Brick * p)
{
    x = X;
    y = Y;
    z = Z;
    next = n;
    prev = p;
}

Brick::~Brick()
{
    if (next == NULL && prev != NULL)
        prev->next = NULL;
    else if (prev == NULL && next != NULL)
        next->prev = NULL;
    else if (next != NULL && prev != NULL)
    {
        next->prev = prev;
        prev->next = next;
    }
}

void Brick::draw()
{
    glPushMatrix();
    glTranslated(x, y, z);

    glColor3ub(230, 130, 120);

    glBegin(GL_QUADS);

    glNormal3i(0,1,0);	//dessus
    glVertex3d(1.5, 1, 0.3);
    glVertex3d(1.5, 1, -0.3);
    glVertex3d(-1.5, 1, -0.3);
    glVertex3d(-1.5, 1, 0.3);

    glNormal3i(0,-1,0); //dessous
    glVertex3d(1.5, -1, 0.3);
    glVertex3d(1.5, -1, -0.3);
    glVertex3d(-1.5, -1, -0.3);
    glVertex3d(-1.5, -1, 0.3);

    glNormal3i(1,0,0); //droite
    glVertex3d(2, 0.6, 0.3);
    glVertex3d(2, -0.6, 0.3);
    glVertex3d(2, -0.6, -0.3);
    glVertex3d(2, 0.6, -0.3);

    glNormal3i(-1,0,0);  //gauche
    glVertex3d(-2, 0.6, -0.3);
    glVertex3d(-2, 0.6, 0.3);
    glVertex3d(-2, -0.6, 0.3);
    glVertex3d(-2, -0.6, -0.3);

    glNormal3i(0,0,1);//devant
    glVertex3d(1.5, 0.6, 0.5);
    glVertex3d(-1.5, 0.6, 0.5); 
    glVertex3d(-1.5, -0.6, 0.5);
    glVertex3d(1.5, -0.6, 0.5);
	
	 glNormal3i(0,0,-1); //derriere
    glVertex3d(1.5, 0.6, -0.5);
    glVertex3d(-1.5, 0.6, -0.5);
    glVertex3d(-1.5, -0.6, -0.5);
    glVertex3d(1.5, -0.6, -0.5);

    glNormal3i(0,1,1); //avant haut
    glVertex3d(1.5, 0.6, 0.5);
    glVertex3d(1.5, 1, 0.3);
    glVertex3d(-1.5, 1, 0.3);
    glVertex3d(-1.5, 0.6, 0.5);

    glNormal3i(0,-1,1); //Avant bas
    glVertex3d(1.5, -0.6, 0.5);
    glVertex3d(1.5, -1, 0.3);
    glVertex3d(-1.5, -1, 0.3);
    glVertex3d(-1.5, -0.6, 0.5);

    glNormal3i(1,0,1); //avant droit
    glVertex3d(1.5, -0.6, 0.5);
    glVertex3d(2, -0.6, 0.3);
    glVertex3d(2, 0.6, 0.3);
    glVertex3d(1.5, 0.6, 0.5);

    glNormal3i(-1,0,1); //avant gauche
    glVertex3d(-1.5, -0.6, 0.5);
    glVertex3d(-2, -0.6, 0.3);
    glVertex3d(-2, 0.6, 0.3);
    glVertex3d(-1.5, 0.6, 0.5);

    glNormal3i(0,1,-1); //arriere haut
    glVertex3d(1.5, 0.6, -0.5);
    glVertex3d(1.5, 1, -0.3);
    glVertex3d(-1.5, 1, -0.3);
    glVertex3d(-1.5, 0.6, -0.5);

    glNormal3i(0,-1,-1); //arriere bas
    glVertex3d(1.5, -0.6, -0.5);
    glVertex3d(1.5, -1, -0.3);
    glVertex3d(-1.5, -1, -0.3);
    glVertex3d(-1.5, -0.6, -0.5);

    glNormal3i(1,0,-1); //arriere droit
    glVertex3d(1.5, -0.6, -0.5);
    glVertex3d(2, -0.6, -0.3);
    glVertex3d(2, 0.6, -0.3);
    glVertex3d(1.5, 0.6, -0.5);

    glNormal3i(-1,0,-1); //arriere gauche
    glVertex3d(-1.5, -0.6, -0.5);
    glVertex3d(-2, -0.6, -0.3);
    glVertex3d(-2, 0.6, -0.3);
    glVertex3d(-1.5, 0.6, -0.5);

    glNormal3i(-1,1,0); //coin haut gauche
    glVertex3d(-2, 0.6, 0.3);
    glVertex3d(-1.5, 1, 0.3);
    glVertex3d(-1.5, 1, -0.3);
    glVertex3d(-2, 0.6, -0.3);

    glNormal3i(-1,-1,0); //coin bas gauche
    glVertex3d(-2, -0.6, 0.3);
    glVertex3d(-1.5, -1, 0.3);
    glVertex3d(-1.5, -1, -0.3);
    glVertex3d(-2, -0.6, -0.3);

    glNormal3i(1,1,0); //coin haut droit
    glVertex3d(2, 0.6, 0.3);
    glVertex3d(1.5, 1, 0.3);
    glVertex3d(1.5, 1, -0.3);
    glVertex3d(2, 0.6, -0.3);

    glNormal3i(1,-1,0); //coin bas droit
    glVertex3d(2, -0.6, 0.3);
    glVertex3d(1.5, -1, 0.3);
    glVertex3d(1.5, -1, -0.3);
    glVertex3d(2, -0.6, -0.3);

    glEnd();
	glBegin(GL_TRIANGLES);

	glNormal3i(-1,1,1);
    glVertex3d(-2, 0.6, 0.3);
    glVertex3d(-1.5, 0.6, 0.5);
    glVertex3d(-1.5, 1, 0.3);

	glNormal3i(1,1,1);
    glVertex3d(2, 0.6, 0.3);
    glVertex3d(1.5, 0.6, 0.5);
    glVertex3d(1.5, 1, 0.3);

	glNormal3i(-1,-1,1);
    glVertex3d(-2, -0.6, 0.3);
    glVertex3d(-1.5, -0.6, 0.5);
    glVertex3d(-1.5, -1, 0.3);

	glNormal3i(1,-1,1);
    glVertex3d(2, -0.6, 0.3);
    glVertex3d(1.5, -0.6, 0.5);
    glVertex3d(1.5, -1, 0.3);


	glNormal3i(-1,1,-1);
    glVertex3d(-2, 0.6, -0.3);
    glVertex3d(-1.5, 0.6, -0.5);
    glVertex3d(-1.5, 1, -0.3);

	glNormal3i(1,1,-1);
    glVertex3d(2, 0.6, -0.3);
    glVertex3d(1.5, 0.6, -0.5);
    glVertex3d(1.5, 1, -0.3);

	glNormal3i(-1,-1,-1);
    glVertex3d(-2, -0.6, -0.3);
    glVertex3d(-1.5, -0.6, -0.5);
    glVertex3d(-1.5, -1, -0.3);

	glNormal3i(1,-1,-1);
    glVertex3d(2, -0.6, -0.3);
    glVertex3d(1.5, -0.6, -0.5);
    glVertex3d(1.5, -1, -0.3);

	glEnd();

    glPopMatrix();
}

void CreateBricks(Brick **first,int quantite)
{
    int i = 1;
    if (quantite >= 1)
    {
        *first = new Brick(-34, 25, 0, NULL, NULL);
        Brick *current = *first;
        Brick *next = NULL;

        while (i < quantite)
        {
            next = new Brick(-34 + (i % 16) * 4.5, 25 - (i / 16) * 3, 0, NULL, current);
            current->next = next;
            current = next;
            i++;
        }
    }
    else
        *first=NULL;
}

void DrawBricks(Brick *first)
{
    Brick *current = first;
    while (current != NULL)    
    {
        current->draw();
        current = current->next;
    }
}

void Destroy(Brick **first,Brick *target)
{
    if (target->prev == NULL)
        *first = target->next;
    delete target;
}
