#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <cstdlib>

#include "../include/ball.h"
#include "../include/player.h"
#include "../include/brick.h"
#include "../include/debris.h"

#define LARGEUR_ECRAN 800
#define HAUTEUR_ECRAN 600

Ball balle;
Player player;
Brick *BrickList;
Debris *DebrisList;

void dessinerRepere(unsigned int echelle = 1)
{
    glPushMatrix();
    glScalef(echelle, echelle, echelle);
    glBegin(GL_LINES);

    glColor3ub(255, 0, 0);      //X en rouge
    glVertex3d(0, 0, 0);
    glVertex3d(1, 0, 0);

    glColor3ub(0, 255, 0);      //Y en vert
    glVertex3d(0, 0, 0);
    glVertex3d(0, 1, 0);

    glColor3ub(0, 0, 255);      //Z en bleu
    glVertex3d(0, 0, 0);
    glVertex3d(0, 0, 1);

    glEnd();
    glPopMatrix();
}

void dessinerBord()
{
    glPushMatrix();
    glBegin(GL_LINES);

    glColor3ub(255, 255, 0);

    glVertex3d(-40, 35, 0);     //dessus
    glVertex3d(40, 35, 0);

    glVertex3d(-40, -40, 0);    //a gauche
    glVertex3d(-40, 35, 0);

    glVertex3d(40, -40, 0);     //a droite
    glVertex3d(40, 35, 0);

    glEnd();
    glPopMatrix();
}

bool ballbatcollision()
{
    if (balle.isgoingdown() 
        && (balle.x - 1 <= player.x + 5) 
        && (balle.x + 1 >= player.x - 5)
        && (balle.y - 1 <= player.y + 0.5) 
        && (balle.y + 1 >= player.y - 0.5))
        return true;
    else
        return false;
}

bool ballbrickcollision()
{
    Brick *current = BrickList;
    Brick *next;
    if(current != NULL)
        next = current->next;

    bool collided = false;
    while (current != NULL)
    {
        if ((balle.x + 1 >= current->x - 2.2)
            && (balle.x - 1 <= current->x + 2.2)
            && ((balle.y - 1 <= current->y + 0.5) 
            && (balle.y + 1 >= current->y - 0.5)))
        {
            collided = true;
            CreateDebris(&DebrisList,current->x,current->y,current->z);
            Destroy(&BrickList,current);
        }
        current = next;
        if(current != NULL)
            next = current->next;
    }

    return collided;
}

void GameHandler()
{
    DebrisCleaner(&DebrisList);

    if(BrickList == NULL)
    {
        CreateBricks(&BrickList,128);
        balle.launched = false;
    }

    if (!balle.launched)
        balle.followbat(player.x, player.y, player.z);
    else
    {
        balle.deplacement('x');
        if (ballbrickcollision())
            balle.brickcollision('x');
        
        balle.deplacement('y');
        if (ballbatcollision())
            balle.batcollision(player.x, player.y, player.z);
        if (ballbrickcollision())
            balle.brickcollision('y');
    }
}

void DrawScene()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW); //affichage en projection orthogonale
    glLoadIdentity();

    gluLookAt(0, 0, 30, 0, 0, 0, 0, 1, 0);      //definition de la camera: position(xyz), orientation(xyz), verticale(xyz)

    GLfloat blanc[4] = { 10.0, 10.0, 10.0, 1.0 };
    GLfloat jaune[4] = { 7.0, 7.0, 0.0, 1.0 };
    GLfloat position0[4] ={0.0,0.0,20.0,1.0};
    GLfloat position1[4] ={balle.x,balle.y,balle.z,1.0};

    glLightfv(GL_LIGHT0, GL_AMBIENT, blanc);
    glLightfv(GL_LIGHT0, GL_DIFFUSE,  blanc );
    glLightfv(GL_LIGHT0, GL_SPECULAR,  blanc);
    glLightfv(GL_LIGHT0, GL_POSITION, position0);
    glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 1.2);

    glLightfv(GL_LIGHT1, GL_DIFFUSE,  jaune);
    glLightfv(GL_LIGHT1, GL_POSITION, position1);
    glLightf(GL_LIGHT1, GL_LINEAR_ATTENUATION, 1.3);

//    dessinerRepere(20);
    dessinerBord();

    balle.draw();
    player.draw();

    DrawBricks(BrickList);

    DrawDebris(DebrisList);

    glFlush();
    SDL_GL_SwapBuffers();

}

int main(int argc, char *argv[])
{
    int start_time;
    bool continuer = true;
    SDL_Event event;

    //init SDL
    SDL_Init(SDL_INIT_VIDEO); //on a besoin de la video
    SDL_WM_SetCaption("GLbricks", NULL); //titre de la fenetre
    SDL_SetVideoMode(LARGEUR_ECRAN, HAUTEUR_ECRAN, 32, SDL_OPENGL);
    SDL_EnableKeyRepeat(1, 1);

    //init opengl
    glEnable(GL_DEPTH_TEST);    //test de profondeur
    glEnable(GL_LIGHTING);  	// Active l'éclairage
    glEnable(GL_NORMALIZE);     // Prends en compte la normale des polygones
    glEnable(GL_COLOR_MATERIAL);//la couleur des polygones est utilisée pour les effets de lumiere

    glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

  	glEnable(GL_LIGHT0); //lampe globale
    glEnable(GL_LIGHT1); //lampe balle

    glMateriali(GL_FRONT, GL_SHININESS, 50);

    glMatrixMode(GL_PROJECTION);        //affichage en projection orthogonale

    glLoadIdentity();
    gluPerspective(100, (double) 4 / 3, 0.1, 1000);     //fov, ratio, near, far

    //boucle principale
    while (continuer)
    {
        start_time = SDL_GetTicks();
        while (SDL_PollEvent(&event))
        {
            switch (event.type)
            {
            case SDL_QUIT:
                continuer = false;
                break;
            case SDL_KEYDOWN:
                switch (event.key.keysym.sym)
                {
                case SDLK_LEFT:
                    player.moveevent('l');
                    break;
                case SDLK_RIGHT:
                    player.moveevent('r');
                    break;
                case SDLK_SPACE:
                    if (!balle.launched)
                        balle.launch();
                    break;
                default:
                    break;
                }
                break;
            }
        }

        GameHandler();
        DrawScene();

        while (SDL_GetTicks() - start_time < 1000 / 60)
        {
            SDL_Delay(1);
        }
    }

    SDL_Quit();

    return 0;
}
